/*
-------------------------------------------------------------------------------
    This file is a part of the VST Preset Generator
    Copyright (C) 2007  Francois Mazen

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License along
    with this program; if not, write to the Free Software Foundation, Inc.,
    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
-------------------------------------------------------------------------------
  Name: vpgfile.cpp
  Author: Francois Mazen
  Date: 28/06/16 19:22
  Description: VpgFile class that represent the list of VST plugins and presets
               loaded by the user.
-------------------------------------------------------------------------------
*/

#include "vpgfile.h"
#include "vstlistmodel.h"
#include "vpgversion.h"

#include <QtXml/QDomDocument>
#include <QFile>
#include <QTextStream>

//-----------------------------------------------------------------------------
VpgFile::ERROR_CODE VpgFile::ReadFile(const QString& filename, VSTListModel& vstListModel)
{
    ERROR_CODE errorCode = ERROR_NO_ERROR;

    //read the file
    QFile file(filename);
    bool isSuccess = file.open(QIODevice::ReadOnly);
    if (!isSuccess)
    {
        errorCode = ERROR_COULD_NOT_OPEN_FILE;
    }

    QDomDocument doc;
    if(isSuccess)
    {
        isSuccess = doc.setContent(&file);
        if(!isSuccess)
        {
            errorCode = ERROR_COULD_NOT_PARSE_FILE;
        }
        file.close();
    }

    QDomElement root;
    if(isSuccess)
    {
        //check data
        root = doc.documentElement();
        if( root.tagName() != "vpg_vst_list" )
        {
            isSuccess = false;
            errorCode = ERROR_INVALID_VPG_FILE;
        }
    }

    QList<FxHeaderStruct*> fxHeaders;
    if(isSuccess)
    {
        //fill the data:
        QDomNode node = root.firstChild();
        while(!node.isNull())
        {
          QDomElement e = node.toElement();
          if(!e.isNull())
          {
              if(e.tagName() == "vst")
              {
                  //create a new header structure
                  FxHeaderStruct* newFxHeader = new FxHeaderStruct;
                  newFxHeader->fxID = e.attribute( "fx_id", "" );
                  QVariant fxVersion = e.attribute( "fx_version", "" );
                  newFxHeader->fxVersion = fxVersion.toInt();
                  newFxHeader->name = e.attribute( "name", "");
                  QVariant nbParam = e.attribute( "nb_param", "");
                  newFxHeader->nParams = nbParam.toInt();
                  QVariant isChunk = e.attribute( "chunk", QString("false"));
                  newFxHeader->useChunk = isChunk.toBool();
                  newFxHeader->fxPath = e.attribute( "fx_path", "");

                  //stock the structure in our list:
                  fxHeaders.append(newFxHeader);
              }
              else
              {
                  isSuccess = false;
                  errorCode = ERROR_UNKNOWN_XML_TAG;
              }
          }
          else
          {
               isSuccess = false; //check error while reading
               errorCode = ERROR_INVALID_XML_ELEMENT;
          }

          node = node.nextSibling();
        }
    }

    if(isSuccess)
    {
        vstListModel.beginModelChangeWithoutRefresh();

        vstListModel.clearListWithoutRefresh();
        for(int index = 0; index < fxHeaders.size(); index++)
        {
            vstListModel.addPluginHeaderWithoutRefresh(fxHeaders[index]);
        }

        vstListModel.endModelChangeWithoutRefresh();
    }

    return errorCode;
}

//-----------------------------------------------------------------------------
VpgFile::ERROR_CODE VpgFile::WriteFile(const QString& filename, const VSTListModel& vstListModel)
{
    ERROR_CODE errorCode = ERROR_NO_ERROR;

    //create the dom (virtual) file
    QDomDocument doc;
    QDomElement root = doc.createElement( "vpg_vst_list" );
    root.setAttribute("major_number", VpgVersion::getCurrentVersion().getMajorNumber());
    root.setAttribute("minor_number", VpgVersion::getCurrentVersion().getMinorNumber());
    root.setAttribute("revision_number", VpgVersion::getCurrentVersion().getRevisionNumber());
    doc.appendChild( root );

    for(int i=0; i < vstListModel.getPluginHeaders().size(); i++)
    {
        FxHeaderStruct* struct_temp = vstListModel.getPluginHeaders().value(i);
        QDomElement cn = doc.createElement( "vst" );
        cn.setAttribute( "name", struct_temp->name );
        cn.setAttribute( "fx_id", struct_temp->fxID );
        cn.setAttribute( "fx_version", struct_temp->fxVersion );
        cn.setAttribute( "nb_param", struct_temp->nParams );
        cn.setAttribute( "chunk", struct_temp->useChunk );
        cn.setAttribute( "fx_path", struct_temp->fxPath );

        root.appendChild(cn);
    }

    QFile file( filename );
    if(!file.open( QIODevice::WriteOnly ))
    {
        errorCode = ERROR_COULD_NOT_OPEN_FILE;
    }
    else
    {
        QTextStream textStream( &file );
        textStream << doc.toString();
        file.close();
    }

    return errorCode;
}
